# This file contains useful module functions.

#--
################################################################################
#                      Copyright (C) 2011 Travis Herrick                       #
################################################################################
#                                                                              #
#                                 \v^V,^!v\^/                                  #
#                                 ~%       %~                                  #
#                                 {  _   _  }                                  #
#                                 (  *   -  )                                  #
#                                 |    /    |                                  #
#                                  \   _,  /                                   #
#                                   \__.__/                                    #
#                                                                              #
################################################################################
# This program is free software: you can redistribute it                       #
# and/or modify it under the terms of the GNU General Public License           #
# as published by the Free Software Foundation,                                #
# either version 3 of the License, or (at your option) any later version.      #
#                                                                              #
# Commercial licensing may be available for a fee under a different license.   #
################################################################################
# This program is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY;                                                    #
# without even the implied warranty of MERCHANTABILITY                         #
# or FITNESS FOR A PARTICULAR PURPOSE.                                         #
# See the GNU General Public License for more details.                         #
#                                                                              #
# You should have received a copy of the GNU General Public License            #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.        #
################################################################################
#++

# Useful functions for modules.
module ModuleFunctions
  class << self
    # Imports public methods from one module into another.
    # ==== Input
    # [receiver : Module] This module will have new methods added to it.
    # [sender : Module] This module will have its methods
    #                   called from <tt>receiver</tt>.
    def import_public_methods(receiver, sender)
      unless receiver.class == Module && sender.class == Module
        raise ArgumentError, "Invalid parameter for #{__method__}. " +
          "A #{Module} was expected, but was a #{receiver.class}."
      end

      # Create public methods of the same name in the receiving module
      # that will call the original module's methods.
      (sender.public_methods - sender.class.public_methods).each do |method|
        receiver.module_eval(<<-EOT, __FILE__, __LINE__)
          def self.#{method}(*args)
            if args.length == 0
              #{sender.name}.#{method}
            else
              #{sender.name}.#{method}(*args)
            end
          end
        EOT
      end
    end
  end
end
